@@ -4,7 +4,7 @@ use simplelog::{
 use std::env::args;
 use std::net::SocketAddr;
 
-use chamomile::prelude::{start, Config, ReceiveMessage, SendMessage};
+use chamomile::prelude::{start, Config, Peer, ReceiveMessage, SendMessage};
 
 #[tokio::main]
 async fn main() {
@@ -22,7 +22,7 @@ async fn main() {
         .parse()
         .expect("invalid addr");
 
-    let mut config = Config::default(self_addr);
+    let mut config = Config::default(Peer::socket(self_addr));
     config.permission = true;
 
     let (peer_id, send, mut recv) = start(config).await.unwrap();
@@ -31,7 +31,7 @@ async fn main() {
     if args().nth(2).is_some() {
         let remote_addr: SocketAddr = args().nth(2).unwrap().parse().expect("invalid addr");
         println!("start connect to remote: {}", remote_addr);
-        send.send(SendMessage::Connect(remote_addr))
+        send.send(SendMessage::Connect(Peer::socket(remote_addr)))
             .await
             .expect("channel failure!");
     }
@@ -41,20 +41,17 @@ async fn main() {
             ReceiveMessage::Data(peer_id, bytes) => {
                 println!("Recv data from: {}, {:?}", peer_id.short_show(), bytes);
             }
-            ReceiveMessage::StableConnect(peer_id, join_data) => {
-                println!("Peer join: {:?}, join data: {:?}", peer_id, join_data);
-                send.send(SendMessage::StableResult(0, peer_id, true, false, vec![1]))
+            ReceiveMessage::StableConnect(peer, join_data) => {
+                println!("Peer join: {:?}, join data: {:?}", peer, join_data);
+                send.send(SendMessage::StableResult(0, peer, true, false, vec![1]))
                     .await
                     .expect("channel failure!");
             }
-            ReceiveMessage::StableResult(peer_id, is_ok, data) => {
-                println!(
-                    "Peer Join Result: {:?} {}, data: {:?}",
-                    peer_id, is_ok, data
-                );
+            ReceiveMessage::StableResult(peer, is_ok, data) => {
+                println!("Peer Join Result: {:?} {}, data: {:?}", peer, is_ok, data);
             }
             ReceiveMessage::ResultConnect(from, _data) => {
-                println!("Recv Result Connect {}", from.to_hex());
+                println!("Recv Result Connect {:?}", from);
             }
             ReceiveMessage::StableLeave(peer_id) => {
                 println!("Peer_leave: {:?}", peer_id);
